/*----------------------------------------------------------------------------

	GPLv3 LICENSE INFORMATION

	This file is part of free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License (version 3) as
	published by the Free Software Foundation. This program is distributed in
	the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the
	implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	You should have received a copy of the GPLv3 along with this software. For
	detailed information, please refer to this copy of the license. Do not remove
	or change this text.

	Preamble
	The following C++ code has been developed based on the MATLAB version (the
	original GUTS prototype software) written in MATLAB by Tjalling Jager
	(http://www.debtox.info/byom.html). The project objective was to translate the
	MATLAB code into C++ code sticking as closely as possible to the original
	MATLAB code and data structures. Also the same names of variables and
	functions were used as in the MATLAB code. Therefore, parts of the present
	code might look slightly complicated and some are redundant. Also comments in
	the code are mostly taken from the MATLAB version.

----------------------------------------------------------------------------*/

//---------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "BatchPrediction_Dialog.h"
//---------------------------------------------------------------------
#pragma resource "*.dfm"
TBatchPredictionDlg *BatchPredictionDlg;
//---------------------------------------------------------------------
__fastcall TBatchPredictionDlg::TBatchPredictionDlg(TComponent* AOwner)
	: TForm(AOwner)
{
}
//---------------------------------------------------------------------
void __fastcall TBatchPredictionDlg::FormCreate(TObject *Sender)
{
	FileListBox->Clear();

	AddFileOpenDialog->DefaultExt = INPUT_DATA_FILE_EXT;
	AddFileOpenDialog->Filter = "Text file (*." + INPUT_DATA_FILE_EXT + ")|*." +
		INPUT_DATA_FILE_EXT + "|All Files (*.*)|*.*";
	AddFileOpenDialog->FilterIndex = 0;

	BatchProgressBar->Visible = false;
	BatchProgressLabel->Caption = "";

    return;
}
//---------------------------------------------------------------------------

void __fastcall TBatchPredictionDlg::AddButtonClick(TObject *Sender)
{
	if (!AddFileOpenDialog->Execute())
		return;

	for (int f=0;f<AddFileOpenDialog->Files->Count; f++) {
		FileListBox->Items->Add(AddFileOpenDialog->Files->Strings[f]);
	}
}
//---------------------------------------------------------------------------

void TBatchPredictionDlg::DisableBatchDialog()
{
	UpperPanel->Enabled = false;
	OKBtn->Enabled = false;
	RunBtn->Enabled = false;
	StopBatchButton->Enabled = true;
	Application->ProcessMessages();

	return;
}
//---------------------------------------------------------------------------

void TBatchPredictionDlg::EnableBatchDialog()
{
	UpperPanel->Enabled = true;
	OKBtn->Enabled = true;
	RunBtn->Enabled = true;
	StopBatchButton->Enabled = false;
	Application->ProcessMessages();

	return;
}
//---------------------------------------------------------------------------

void __fastcall TBatchPredictionDlg::RunBtnClick(TObject *Sender)
{
	// check if batch predictions can be performed
    if (!MainForm->CalibrationPerformed)
	{
		MessageDlg("A prediction can only be performed after a successful calibration of the (unchanged) input data sets! Please load a project file containing calibration results or conduct a calibration with suitable data.", mtInformation, TMsgDlgButtons() << mbOK, 0);
		return;
	}

	if (FileListBox->Items->Count<1)
	{
		MessageDlg("Please select at least one file!", mtInformation, TMsgDlgButtons() << mbOK, 0);
		return;
	}
/*
	OutputFolderEdit->Text=Trim(OutputFolderEdit->Text);
	if (!DirectoryExists(OutputFolderEdit->Text))
	{
		MessageDlg("Please select an output folder!", mtInformation, TMsgDlgButtons() << mbOK, 0);
		return;
	}
*/
	if (!DoSDPredictionsCheckBox->Checked && !DoITPredictionsCheckBox->Checked)
	{
		MessageDlg("Please select at least one GUTS model to predict!", mtInformation, TMsgDlgButtons() << mbOK, 0);
		return;
	}

/*	if (CountProcesses("WINWORD.EXE") > 0)
	{
		AnsiString as = (AnsiString)"Warning: All instances of MS Word have to be closed before running batch predictions" +
						" with subsequent report creation." +
						"\n\nPress 'OK' to proceed (CHANGES IN ACTIVE DOCUMENTS WILL BE LOST)." +
						"\nPress 'Cancel' to close all instances of MS Word manually.";

		if (MessageDlg(as, mtWarning, TMsgDlgButtons() << mbOK << mbCancel, 0) != mrOk)
			return;
		else
			MainForm->KillMSWordProcesses();
	}*/

	// disable dialog
	DisableBatchDialog();

	MainForm->DoITBatchPredictions = DoITPredictionsCheckBox->Checked;
	MainForm->DoSDBatchPredictions = DoSDPredictionsCheckBox->Checked;

	MainForm->BatchFiles.clear();
	MainForm->BatchOutputFolder = c_str(OutputFolderEdit->Text);
	for (int f=0;f<FileListBox->Items->Count; f++)
		MainForm->BatchFiles.push_back(c_str(FileListBox->Items->Strings[f]));

	BatchProgressBar->Position = 0;
	BatchProgressBar->Visible = true;
	Application->ProcessMessages();

	BatchMemo->Clear();
	BatchMemo->Lines->Add("Starting batch processing ...");
    AnsiString err;
	MainForm->RunBatch(err);
	if (err != "")
	{
		Application->ProcessMessages();
		EnableBatchDialog();
		return;
	}

	Application->ProcessMessages();

	EnableBatchDialog();

	return;
}
//---------------------------------------------------------------------------

void __fastcall TBatchPredictionDlg::DeleteButtonClick(TObject *Sender)
{
	FileListBox->DeleteSelected();
}
//---------------------------------------------------------------------------

void __fastcall TBatchPredictionDlg::ClearButtonClick(TObject *Sender)
{
	while (FileListBox->Items->Count > 0)
		FileListBox->Items->Delete(0);
}
//---------------------------------------------------------------------------


void __fastcall TBatchPredictionDlg::SpeedButton1Click(TObject *Sender)
{
	if (!SetFolderFileOpenDialog->Execute())
		return;

	OutputFolderEdit->Text = SetFolderFileOpenDialog->Files->Strings[0];
}
//---------------------------------------------------------------------------

void __fastcall TBatchPredictionDlg::DoSDPredictionsCheckBoxClick(TObject *Sender)

{
	MainForm->DoSDBatchPredictions = DoSDPredictionsCheckBox->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TBatchPredictionDlg::DoITPredictionsCheckBoxClick(TObject *Sender)

{
	MainForm->DoITBatchPredictions = DoITPredictionsCheckBox->Checked;
}
//---------------------------------------------------------------------------

void __fastcall TBatchPredictionDlg::StopBatchButtonClick(TObject *Sender)
{
	MainForm->StopBatchPredictions = true;
}
//---------------------------------------------------------------------------

void __fastcall TBatchPredictionDlg::FormShow(TObject *Sender)
{
	BatchProgressBar->Visible = false;
}
//---------------------------------------------------------------------------

